@@ -56,11 +56,11 @@ if ($^O eq 'cygwin') {
     if (!defined(&ExtUtils::MM_Cygwin::maybe_command)) {
         *ExtUtils::MM_Cygwin::maybe_command = sub {
             my ($self, $file) = @_;
-            if ($file =~ m{^/cygdrive/}i) {
+            if ($file =~ m{^/cygdrive/}i and ExtUtils::MM_Win32->can('maybe_command')) {
                 ExtUtils::MM_Win32->maybe_command($file);
             }
             else {
-                $self->SUPER::maybe_command($file);
+                ExtUtils::MM_Unix->maybe_command($file);
             }
         }
     }
